@@ -1,14 +1,16 @@
 """
 node.py - basic nodeid manipulation for mercurial
 
-Copyright 2005 Matt Mackall <mpm@selenic.com>
+Copyright 2005, 2006 Matt Mackall <mpm@selenic.com>
 
 This software may be used and distributed according to the terms
 of the GNU General Public License, incorporated herein by reference.
 """
 
-import binascii
+from demandload import demandload
+demandload(globals(), "binascii")
 
+nullrev = -1
 nullid = "\0" * 20
 
 def hex(node):
